@@ -34,3 +34,9 @@ fetch_chrom_sizes_from_UCSC <- function(genome,
     ans
 }
 
+read_UCSC_assembled_molecules_info_table <- function(file)
+{
+    col2class <- c(chrom="character", size="integer", circular="logical")
+    simple_read_table(file, header=TRUE, col2class=col2class)
+}
+
