@@ -143,6 +143,7 @@ test_seqlevelsStyle_Seqinfo <- function()
         list("canFam3",  "CanFam3.1",                  3268L,    0L,      0L),
         list("canFam4",  "UU_Cfam_GSD_1.0",            2198L,    0L,      0L),
         list("canFam5",  "UMICH_Zoey_3.1",              794L,    0L,      0L),
+        list("canFam6",  "Dog10K_Boxer_Tasha",          147L,    0L,      0L),
         list("ce6",      "WS190",                         7L,    0L,      0L),
         list("ce10",     "WBcel215",                      7L,    0L,      0L),
         list("ce11",     "WBcel235",                      7L,    0L,      0L),
@@ -151,6 +152,9 @@ test_seqlevelsStyle_Seqinfo <- function()
         list("danRer11", "GRCz11",                     1923L,    0L,      0L),
         list("dm3",      "Release 5",                    14L,    1L,      0L),
         list("dm6",      "Release 6 plus ISO1 MT",     1870L,    0L,      0L),
+        list("equCab2",  "EquCab2.0",                    33L,    1L,   9604L),
+        list("equCab3",  "EquCab3.0",                  4701L,    0L,      0L),
+        list("felCat9",  "Felis_catus_9.0",            4508L,    0L,      0L),
         list("galGal3",  "Gallus_gallus-2.1",            34L,   23L,  17118L),
         list("galGal4",  "Gallus_gallus-4.0",         15932L,    0L,      0L),
         list("galGal5",  "Gallus_gallus-5.0",         23475L,    0L,      0L),
@@ -161,16 +165,13 @@ test_seqlevelsStyle_Seqinfo <- function()
         #list("hg18",     "NCBI36",                       26L,   23L,     97L),
         list("hg19",     "GRCh37.p13",                  297L,    1L,      0L),
         list("hg38",     "GRCh38.p13",                  640L,    0L,      0L),
+        list("hs1",      "T2T-CHM13v2.0",                25L,    0L,      0L),
         list("macFas5",  "Macaca_fascicularis_5.0",    7601L,    0L,      0L),
-        list("rheMac2",  "Mmul_051212",                  21L,    1L, 122143L),
-        list("rheMac3",  "CR_1.0",                    34102L,    1L,      0L),
-        list("rheMac8",  "Mmul_8.0.1",               284728L,    0L,      0L),
-        list("rheMac10", "Mmul_10",                    2939L,    0L,      0L),
-        list("monDom5",  "MonDom5",                      10L,    1L,   5006L),
         list("mm8",      "MGSCv36",                      21L,   13L,    360L),
         list("mm9",      "MGSCv37",                      22L,   13L,    283L),
         list("mm10",     "GRCm38.p6",                   239L,    0L,      0L),
         list("mm39",     "GRCm39",                       61L,    0L,      0L),
+        list("monDom5",  "MonDom5",                      10L,    1L,   9028L),
         list("musFur1",  "MusPutFur1.0",               7741L,    0L,     42L),
         list("panPan1",  "panpan1",                   10867L,    0L,      0L),
         list("panPan2",  "panpan1.1",                 10274L,    0L,      0L),
@@ -180,6 +181,10 @@ test_seqlevelsStyle_Seqinfo <- function()
         list("panTro4",  "Pan_troglodytes-2.1.4",     24129L,    0L,      0L),
         list("panTro5",  "Pan_tro 3.0",               44449L,    0L,      0L),
         list("panTro6",  "Clint_PTRv2",                4346L,    0L,      0L),
+        list("rheMac2",  "Mmul_051212",                  21L,    1L, 122143L),
+        list("rheMac3",  "CR_1.0",                    34102L,    1L,      0L),
+        list("rheMac8",  "Mmul_8.0.1",               284728L,    0L,      0L),
+        list("rheMac10", "Mmul_10",                    2939L,    0L,      0L),
         list("rn5",      "Rnor_5.0",                   2739L,    0L,      0L),
         list("rn6",      "Rnor_6.0",                    953L,    0L,      2L),
         list("rn7",      "mRatBN7.2",                   176L,    0L,      0L),
@@ -187,7 +192,8 @@ test_seqlevelsStyle_Seqinfo <- function()
         list("susScr2",  "Sscrofa9.2",                   19L,    1L,      0L),
         list("susScr3",  "Sscrofa10.2",                4583L,    0L,      0L),
         list("susScr11", "Sscrofa11.1",                 613L,    0L,      0L),
-        list("taeGut2",  "Taeniopygia_guttata-3.2.4", 37096L,    0L,      0L)
+        list("taeGut2",  "Taeniopygia_guttata-3.2.4", 37096L,    0L,      0L),
+        list("xenTro10", "UCB_Xtro_10.0",               167L,    0L,      0L)
     )
     for (i in seq_along(UCSC_NCBI)) {
         args <- UCSC_NCBI[[i]]
@@ -265,10 +271,12 @@ test_seqlevelsStyle_Seqinfo <- function()
     }
 
     ## Exclude some genomes from the RefSeq switch check. These genomes
-    ## fail to pass the check for reasons that need to be investigated
-    ## (in the case of canFam5 it's because chrM is not mapped to a RefSeq
-    ## accession).
-    skip_RefSeq_switch <- c("canFam4", "canFam5", "rheMac3", "panTro3")
+    ## fail to pass the check for reasons that need to be investigated!
+    ## In the case of canFam5 and hs1, it's because chrM is not mapped
+    ## to a RefSeq accession, that is, RefSeqAccn is NA for chrM in
+    ## getChromInfoFromNCBI("UMICH_Zoey_3.1"), and for MT in
+    ## getChromInfoFromNCBI("T2T-CHM13v2.0").
+    skip_RefSeq_switch <- c("canFam4", "canFam5", "hs1", "rheMac3", "panTro3")
     for (i in seq_along(UCSC_NCBI)) {
         args <- UCSC_NCBI[[i]][c(1L, 2L, 4L)]
         if (args[[1L]] %in% skip_RefSeq_switch)
